package org.xiss.index;

/**
 * An abstract interface being the base for both element/attribute index entries
 * and structure index entries. This interface by itself is useless.
 */
public interface IIndexEntry {
	/**
	 * Denotes a 'null' value for node identifiers, for instance a node with no
	 * parent will have ParentId = NONE.
	 */
	public static final int NONE = -1;

	/**
	 * Get ID of the entry (number of its corresponding element in preorder
	 * traversal)
	 */
	public int getId();

	/**
	 * Get size of the entry (total count of its subnodes)
	 */
	public int getSize();

	/**
	 * Get id of the first node not contained in this one. Is equivalent to
	 * getId()+getSize().
	 */
	public int getNextId();

	/**
	 * Get ID of the parent of this node. If this is the root node, the return
	 * value is {@link #NONE}
	 */
	public int getParentId();

	/**
	 * Get ID of the value of this node (key for value table) For attributes
	 * this means value of the attribute, for text nodes this means their text
	 * value, otherwise the value is {@link #NONE}.
	 */
	public int getValueId();
}
