package org.xiss.index.impl.persistence;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.List;

import net.sf.xanswer.storage.map.PersistentException;

public class PersistentList<T> extends AbstractList<T> {
	private List<T> myActualList;

	private File myFile;

	public PersistentList() {
		myActualList = new ArrayList<T>();
	}

	public T get(int index) {
		return myActualList.get(index);
	}

	public int size() {
		return myActualList.size();
	}

	public T set(int index, T value) {
		myActualList.set(index, value);
		return value;
	}

	public void add(int index, T element) {
		myActualList.add(index, element);
	}

	public T remove(int index) {
		return myActualList.remove(index);
	}

	public void create(String directory, String file) {
		myFile = new File(directory, file);
	}

	public void createNew() throws PersistentException {
		try {
			if (myFile.exists()) {
				myFile.delete();
				myFile.createNewFile();
			}
		} catch (IOException e) {
			throw new PersistentException(e);
		}
	}

	public void open() throws PersistentException {
		InputStream s = null;
		try {
			s = new FileInputStream(myFile);
			ObjectInputStream stream = new ObjectInputStream(s);

			myActualList = (List<T>) stream.readObject();
		} catch (FileNotFoundException e) {
			throw new PersistentException(e);
		} catch (IOException e) {
			throw new PersistentException(e);
		} catch (ClassNotFoundException e) {
			throw new PersistentException(e);
		} finally {
			if (s != null) {
				try {
					s.close();
				} catch (IOException e) {
				}
			}
		}
	}

	public void close() throws PersistentException {
		OutputStream s = null;
		try {
			s = new FileOutputStream(myFile);
			ObjectOutputStream stream = new ObjectOutputStream(s);

			stream.writeObject(myActualList);
		} catch (FileNotFoundException e) {
			throw new PersistentException(e);
		} catch (IOException e) {
			throw new PersistentException(e);
		} finally {
			if (s != null) {
				try {
					s.close();
				} catch (IOException e) {
				}
			}
		}
	}

	public void drop() {
		myFile.delete();
	}
}
