package org.xiss.query.impl.ast;

import java.util.List;

import org.xiss.query.IXissOperation;
import org.xiss.query.operations.JoinOperation;
import org.xiss.query.operations.SelectRootOperation;

public class GlobalPathNode extends AstNode {
	private PathNode myPath;

	private boolean myStartsFromRoot;

	public GlobalPathNode(PathNode path, boolean startsFromRoot) {
		myPath = path;
		myStartsFromRoot = startsFromRoot;
	}

	public void generateCodeTo(List<IXissOperation> script) {
		// The one and only special case is when the path starts
		// with a wildcard: _[k..inf]/<something else>, or the same
		// but starting from root: /_k/...
		// This case should be handled separately because a wildcard
		// doesn't have its own code generation.

		boolean isExactJoin;
		int joinDepth;

		if (myPath != null && myPath.getLeft() != null
				&& myPath.getLeft().getCore() instanceof AnyNode
				&& myPath.getLeft().getPredicate() == null) {
			AnyNode any = (AnyNode) myPath.getLeft().getCore();
			// So, in this case:
			// If the path doesn't start from root and 'any' has a low-bounded
			// count, we still must join the right result on the root node.
			// If it doesn't start from root and 'any' has a zero lower bound,
			// simply leave out the wildcard.
			// If it starts from root,

			if (!any.isExact() && any.getCount() == 0) {// /_*/...
				if (myStartsFromRoot) {
					script.add(new SelectRootOperation());
				}

				if (myPath.getRight() != null) {
					myPath.getRight().generateCodeTo(script);
				}

				if (myStartsFromRoot) {
					script.add(new JoinOperation(1, false));
				}
				return;
			}

			joinDepth = any.getCount();
			if (myStartsFromRoot) {
				isExactJoin = any.isExact();
			} else {
				isExactJoin = false;
			}
		} else {
			joinDepth = 1;
			isExactJoin = true;
		}

		if (myStartsFromRoot) {
			script.add(new SelectRootOperation());
		}

		if (myPath != null) {
			myPath.generateCodeTo(script);

			if (myStartsFromRoot) {
				script.add(new JoinOperation(joinDepth, isExactJoin));
			}
		}
	}

	public void preprocessForGeneration() {
		if (myPath != null) {
			myPath.preprocessForGeneration();
		}
	}
}
