package org.xiss.query.impl.ast;

import java.util.List;

import org.xiss.query.IXissOperation;
import org.xiss.query.operations.JoinOperation;

public class PathNode extends AstNode {
	private PathChunkNode myLeft;

	private PathNode myRight;

	public PathNode(PathChunkNode onlyChunk) {
		setLeft(onlyChunk);
	}

	public PathNode(PathChunkNode left, PathNode right) {
		setLeft(left);
		setRight(right);
	}

	public void preprocessForGeneration() {
		getLeft().preprocessForGeneration();
		if (getRight() != null) {
			getRight().preprocessForGeneration();
		}
		kleeneCloseWildcards();
		compressWildcards();
	}

	private void kleeneCloseWildcards() {
		if (getLeft().getCore() instanceof AnyNode
				&& getLeft().getPredicate() == null) {
			if (getLeft().isKleeneClosure()) {
				AnyNode left = (AnyNode) (getLeft().getCore());
				if (left.getCount() != 1) {
					throw new RuntimeException(
							"Kleene closures of multiple AnyNode's not supported");
				}
				left.setExact(false);
				left.setCount(0);
				getLeft().setKleeneClosure(false);
			}
		}
	}

	private void compressWildcards() {

		// _a/_b/... -> _(a+b)/...
		if (getLeft().getCore() instanceof AnyNode
				&& getLeft().getPredicate() == null) {
			if (getRight() == null) {
				return;
			}
			PathChunkNode rleft = getRight().getLeft();
			if (rleft.getCore() instanceof AnyNode
					&& rleft.getPredicate() == null) {
				AnyNode left = (AnyNode) (getLeft().getCore());
				int leftCount = left.getCount();

				AnyNode right = (AnyNode) (rleft.getCore());
				int rightCount = right.getCount();

				left.setCount(leftCount + rightCount);
				left.setExact(left.isExact() && right.isExact());

				setRight(getRight().getRight());
			}
		}
	}

	public void generateCodeTo(List<IXissOperation> script) {
		getLeft().generateCodeTo(script);
		if (getRight() != null) {
			PathChunkNode rleft = getRight().getLeft();
			if (rleft.getCore() instanceof AnyNode
					&& rleft.getPredicate() == null) {
				AnyNode right = (AnyNode) (rleft.getCore());
				int rightCount = right.getCount();

				getRight().getRight().generateCodeTo(script);
				script.add(new JoinOperation(rightCount + 1, right.isExact()));
			} else {
				getRight().generateCodeTo(script);
				script.add(new JoinOperation(1, true));
			}
		}
	}

	public void setLeft(PathChunkNode left) {
		myLeft = left;
	}

	public PathChunkNode getLeft() {
		return myLeft;
	}

	public void setRight(PathNode right) {
		myRight = right;
	}

	public PathNode getRight() {
		return myRight;
	}
}
