package org.xiss.query.operations;

import java.util.Stack;

import org.xiss.index.IXissIndex;
import org.xiss.query.IResult;
import org.xiss.query.IXissOperation;
import org.xiss.query.impl.ChunkList;

/**
 * Computes the Kleene closure of the result on top of the evaluation stack
 */
public class KleeneClosureOperation implements IXissOperation {
	public KleeneClosureOperation() {
	}

	public void run(IXissIndex index, Stack<IResult> context) {
		IResult topResult = context.peek();
		IResult cur = topResult.copy();
		IResult init = topResult.copy();
		while (!cur.isEmpty()) {
			context.push(new ChunkList(cur.getChunks()));
			context.push(new ChunkList(init.getChunks()));

			new JoinOperation(1, true).run(index, context);
			cur = context.peek();

			new UnionOperation().run(index, context);
		}
	}

	public String toString() {
		return "KC";
	}
}
