package org.xiss.query.operations;

import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.xiss.index.IXissIndex;
import org.xiss.query.Chunk;
import org.xiss.query.IResult;
import org.xiss.query.IXissOperation;
import org.xiss.query.impl.ChunkList;

/**
 * Computes the union of two results
 */
public class UnionOperation implements IXissOperation {
	public UnionOperation() {
	}

	public void run(IXissIndex index, Stack<IResult> context) {
		IResult rightRes = context.pop(), leftRes = context.pop();
		ChunkList res = new ChunkList();
		boolean sortRootNotDeep;
		if (leftRes.isSortedByStart()) {
			if (!rightRes.isSortedByStart()) {
				rightRes.sortByStart();
			}
			res.setRootSorted();
			sortRootNotDeep = true;
		} else if (leftRes.isSortedByEnd()) {
			if (!rightRes.isSortedByEnd()) {
				rightRes.sortByEnd();
			}
			res.setDeepSorted();
			sortRootNotDeep = false;
		} else {
			leftRes.sortByStart();
			rightRes.sortByStart();
			res.setRootSorted();
			sortRootNotDeep = true;
		}

		List<Chunk> leftChunks = leftRes.getChunks();
		List<Chunk> rightChunks = rightRes.getChunks();

		Iterator<Chunk> li = leftChunks.listIterator();
		Iterator<Chunk> ri = rightChunks.listIterator();

		Chunk left = null;
		Chunk right = null;
		if (li.hasNext()) {
			left = li.next();
		} else {
			left = null;
		}
		if (ri.hasNext()) {
			right = ri.next();
		} else {
			right = null;
		}

		while (left != null || right != null) {

			int cmp;
			if (left == null) {
				cmp = 1;
			} else if (right == null) {
				cmp = -1;
			} else {
				cmp = Chunk.compare(left, right, sortRootNotDeep);
			}

			if (cmp < 0) {
				res.addChunk(left);
				left = li.hasNext() ? li.next() : null;
			} else if (cmp > 0) {
				res.addChunk(right);
				right = ri.hasNext() ? ri.next() : null;
			} else {
				res.addChunk(left);
				if (!left.equals(right)) {
					res.addChunk(right);
				}
				left = li.hasNext() ? li.next() : null;
				right = ri.hasNext() ? ri.next() : null;
			}
		}
		context.push(res);
	}

	public String toString() {
		return "UNION";
	}
}
