package org.xiss.query;

import java.io.File;

import junit.framework.Assert;
import junit.framework.TestCase;
import net.sf.xanswer.indexing.IndexException;

import org.xiss.index.IXissIndex;
import org.xiss.index.XissIndexFactory;

public class QueryRunTest extends TestCase {
	public void testSelect() {
		IXissIndex index = null;
		try {
			index = createIndex("TestXmls/books.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();

			IResult res = factory.parse("book").run(index);
			Assert.assertEquals(res.toString(), "[6..6, 12..12, 17..17]");
			res = factory.parse("isbn:book").run(index);
			Assert.assertEquals(res.toString(), "[22..22]");
			res = factory.parse("/isbn:book").run(index);
			Assert.assertEquals(res.toString(), "[0..22]");
			res = factory.parse("library").run(index);
			Assert.assertEquals(res.toString(), "[0..0]");
		} catch (IndexException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testSelectText() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/normal2.xml");
			XissQueryFactory factory = XissQueryFactory.newInstance();

			IQuery q;
			IResult res;

			q = factory.parse("property[@name=\"Author\"]/text()");
			res = q.run(index);
			Assert.assertEquals(res.toString(), "[49..51, 65..67]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testJoinOnBooks() {
		IXissIndex index = null;
		try {
			index = createIndex("TestXmls/books.xml");
			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			IQuery q = factory.parse("/_*/book");
			res = q.run(index);
			Assert.assertEquals(res.toString(), "[0..6, 0..12, 0..17]");
			res = factory.parse("library/book").run(index);
			Assert.assertEquals(res.toString(), "[0..6, 0..12, 0..17]");
			res = factory.parse("/library/_/book").run(index);
			Assert.assertEquals(res.toString(), "[]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testJoinNormal() {
		IXissIndex index = null;
		try {
			index = createIndex("TestXmls/normal2.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			res = factory.parse("instance/_/img").run(index);
			Assert.assertEquals(res.toString(), "[62..76, 80..90]");

			res = factory.parse("instance/img").run(index);
			Assert.assertEquals(res.toString(), "[]");
			res = factory.parse("instance/_*/img").run(index);
			Assert.assertEquals(res.toString(), "[62..76, 80..90]");
			res = factory.parse("instance/_/_/img").run(index);
			Assert.assertEquals(res.toString(), "[]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testJoin2() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/SimpleKleeneTest.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			res = factory.parse("b").run(index);
			Assert.assertEquals(res.toString(), "[2..2, 5..5, 8..8, 11..11]");
			res = factory.parse("b/b").run(index);
			Assert.assertEquals(res.toString(), "[2..5, 5..8, 8..11]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testUnion() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/UnionTest.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;

			res = factory.parse("a").run(index);
			Assert.assertEquals(res.toString(),
					"[2..2, 8..8, 25..25, 34..34, 37..37]");
			res = factory.parse("b").run(index);
			Assert
					.assertEquals(res.toString(),
							"[4..4, 13..13, 17..17, 21..21, 27..27, 29..29, 41..41, 44..44]");
			res = factory.parse("a|b").run(index);
			Assert
					.assertEquals(
							res.toString(),
							"[2..2, 4..4, 8..8, 13..13, 17..17, 21..21, 25..25, 27..27, 29..29, 34..34, 37..37, 41..41, 44..44]");
			res = factory.parse("b|b").run(index);
			Assert
					.assertEquals(res.toString(),
							"[4..4, 13..13, 17..17, 21..21, 27..27, 29..29, 41..41, 44..44]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testKleeneClosure() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/SimpleKleeneTest.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			res = factory.parse("b*").run(index);
			Assert
					.assertEquals(res.toString(),
							"[2..2, 2..11, 2..8, 2..5, 5..5, 5..11, 5..8, 8..8, 8..11, 11..11]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testAttributeFilter1() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/books.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			IQuery q = factory.parse("book[@price=\"20\"]");
			res = q.run(index);
			Assert.assertEquals(res.toString(), "[12..12]");
			q = factory.parse("book[@price]");
			res = q.run(index);
			Assert.assertEquals(res.toString(), "[6..6, 12..12]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testAttributeFilter2() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/normal2.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			IQuery q = factory.parse("/_*/property[@name=\"Author\"]");
			// FIXME: _*[] crashes the parser (a path chunk can't have kleene
			// closure inside it by grammar,
			// although it should. An alternative is to use (_*)[])
			// FIXME: _[] crashes the runner :) Must convert .../_[@a="v"] to
			// deep EA-join
			// and for other predicates use brute force.
			// System.out.println(q);
			res = q.run(index);
			Assert.assertEquals(res.toString(), "[0..9, 0..49, 0..65]");
			// System.out.println(res);
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	public void testAttributeSelect() {
		IXissIndex index = null;
		try {
			index = createIndex("QueryTests/normal2.xml");

			XissQueryFactory factory = XissQueryFactory.newInstance();
			IResult res;
			res = factory.parse("entity/_/_/@name").run(index);
			Assert
					.assertEquals(res.toString(),
							"[3..10, 3..13, 3..16, 3..20, 25..32, 25..35, 25..38, 25..41]");
			res = factory.parse("entity/_*/@name").run(index);
			Assert
					.assertEquals(res.toString(),
							"[3..10, 3..13, 3..16, 3..20, 25..32, 25..35, 25..38, 25..41]");

			res = factory.parse("/_*/@name").run(index);
			res.sortByEnd();
			Assert
					.assertEquals(
							res.toString(),
							"[0..10, 0..13, 0..16, 0..20, 0..32, 0..35, 0..38, 0..41, 0..50, 0..54, 0..58, 0..66, 0..70, 0..74, 0..84, 0..88, 0..94]");

			res = factory.parse("(/entity/_*/@name)|(/instance/_*/@name)").run(
					index);
			res.sortByEnd();
			Assert
					.assertEquals(
							res.toString(),
							"[0..10, 0..13, 0..16, 0..20, 0..32, 0..35, 0..38, 0..41, 0..50, 0..54, 0..58, 0..66, 0..70, 0..74, 0..84, 0..88, 0..94]");

			res = factory.parse("(/(entity|instance)/_*/@name)").run(index);
			Assert
					.assertEquals(
							res.toString(),
							"[0..10, 0..13, 0..16, 0..20, 0..32, 0..35, 0..38, 0..41, 0..50, 0..54, 0..58, 0..66, 0..70, 0..74, 0..84, 0..88, 0..94]");
		} catch (IndexException e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		} catch (QueryParseException e) {
			e.printStackTrace(System.out);
			Assert.fail(e.getMessage());
		} finally {
			if (index != null) {
				try {
					index.close();
				} catch (IndexException e) {
				}
			}
		}
	}

	private IXissIndex createIndex(String path) throws IndexException {
		uglyHackRemoveDatabase("test");
		return XissIndexFactory.newInstance().createNewFromDocument(
				"input/" + path, "test");
	}

	private void uglyHackRemoveDatabase(String name) {
		File f = new File(name);
		deleteRecursive(f);
	}

	private void deleteRecursive(File f) {
		if (!f.exists()) {
			return;
		}
		if (f.isDirectory()) {
			for (File content : f.listFiles()) {
				deleteRecursive(content);
			}
		}
		f.delete();
	}
}
